% Implementation of an inversion algorithm for a TSI SMPS. The inversion 
% algorithm was tested with DMPS data and published in Atmospheric Measurement Techniques Discussion,
% 8, 10283-10317, 2015. If you use this algorithm for a publication,
% please, cite the article above, or when available, the corresponding
% article in Atmospheric Measurement Techniques (expected to be published
% during 2016).
% You need to have GPstuff version 4.5 installed for it to work. 
%
% 
% The two first lines are needed the first time you work with a data
% file, later on they can be skipped. Edit read_raw_data_v2.m so that it
% reads the raw data file you are interested in.
% You will probably need to enter the time constant of the CPC yourself in
% transferDD2011_v2_2.m

read_raw_data_v2
transferDD2011_v2_2()

%%
clear
kernelOneScan
load counts.mat
rawCounts=rawCounts(firstUsedRawCount+(0:nRawCountInt*nCountInt-1),:);


clear kernel kernel_record
%Choose the scans you want to do the inversion for
indexScans=1:14;

nScans=length(indexScans);

[nr,nc]=size(TOneScan);
x=[];
y=[];
T=sparse(nScans*[nr nc]);
for ii=indexScans
xThisScan=xOneScan+[timeStampDOY(ii)*ones(nr,1) zeros(nr,1)];
x=[x; xThisScan];
T((ii-indexScans(1))*nr+(1:nr),(ii-indexScans(1))*nc+(1:nc))=sparse(TOneScan);
y=[y; sum(reshape(rawCounts(:,ii),nRawCountInt,nCountInt))'];
end

z=ones(size(y))*volumeCountInt*1e6; %giving volume in cm^3



addpath inversion_algorithm


% ===== define the model, create model structures, and run inference =====
% Create likelihood and covariance function
lik = lik_countNumber('T', T); 

% Covariance structures and priors for their hyperparameters
% * You can test also different covariances. See the GPstuff manual for
%   different possibilities and how to use them:
%   http://arxiv.org/abs/1206.5754

% covariance structure corresponding to k(u,u') and priors for its
% hyperparameters
gpcf_p = gpcf_matern52('magnSigma2', 10, 'lengthScale', 1, ...
    'magnSigma2_prior', prior_sqrtt('s2',3) , 'lengthScale_prior', prior_t('s2',0.25), 'selectedVariables', 2);
% covariance structure corresponding to k(t,t') and priors for its
% hyperparameters
gpcf_t = gpcf_exp('magnSigma2', 1, 'lengthScale', 0.03, ...
    'magnSigma2_prior', prior_fixed , 'lengthScale_prior', prior_t('s2',0.01), 'selectedVariables', 1);
% covariance structure corresponding to k(u,u')*k(t,t')
gpcf_pt = gpcf_prod('cf', {gpcf_p, gpcf_t});
% covariance structure corresponding to mean \mu ~ N(0, \sigma^2)
gpcfc = gpcf_constant('constSigma2', 10, 'constSigma2_prior', prior_fixed);

% Define the options for the 'stabilized-newton' algorithm
latent_opt.tol = 1e-8;                                  % f optimization tolerance
latent_opt.maxiter = 450;                               % maximum number of iterations in stabilized Newton algorithm

% The Gaussian process (GP) model structure containing the likelihood and
% covariance functions and information concerning inference
%  * jitterSigma2:  Add "jitter" to the diagonal of the covariance function
gp = gp_setDMPS('lik', lik, 'cf', {gpcf_pt gpcfc}, 'latent_opt', latent_opt, 'jitterSigma2', 1e-4);

% Find the maximum a Posterior approximation of the hyperparameters (\theta) and
% construct Laplace approximation for f at the mode of \theta
opt=optimset('TolFun',1e-3,'TolX',1e-3,'Display','iter');  % Set optimization options into a struct
gp=gp_optimDMPS(gp,x,y,'z',z,'opt',opt);                   % optimize
% ===== END: model definition and inference ====


%% Visualize results

% the MAP of the hyperparameters
gp.cf{1}.cf{1}    % The covariance function along the particle size dimension
gp.cf{1}.cf{2}    % The covariance function along the time dimension

% Predict the particle number size distribution at 
% different time points

% figure
% ns=length(unique(x(:,2)));
% timesteps=linspace(min(x(:,1)),max(x(:,1)),nScans);
% for i1=1:nScans
%     subplot(ceil(sqrt(nScans)),ceil(sqrt(nScans)),i1)
%     xt = [repmat(timesteps(i1),ns,1) unique(x(:,2))];
%     [Ef, Varf] = gpla_predDMPS(gp,x,y,xt,'z',z);
%     semilogx(exp10(xt(:,2)),exp(Ef),'k')
%     hold on
%     semilogx(exp10(xt(:,2)),exp(Ef + 2*sqrt(Varf)),'k--')
%     semilogx(exp10(xt(:,2)),exp(Ef - 2*sqrt(Varf)),'k--')
%     ylabel('d{\itN}/dlog_1_0({\itD_p}) [cm^{-3}]')
%     xlabel('{\itD_p} [m]');
% %    semilogx(exp(xt(:,2)),exp(Ef),col)
% %    hold on,
%     %legend(sprintf('time = %.2f',timesteps(i1)))
%     legend(datestr(timesteps(i1),13))
% end


% Examine the predictions at data points
[Ef, Varf] = gpla_predDMPS(gp,x,y,x,'z',z);
figure
scatter(x(:,1),exp10(x(:,2)),3,Ef)
set(gca,'Yscale','log')
xlabel('Time')
ylabel('Particle diameter [m]')
colorbar
datetick('x')
title('Fitted values of f')


%Compare lambda with actual counts. The closer lambda and y are to each
%other, the better the model fit is. Note that there may be zero counts
%which are not seen because of the logarithmic scale.
figure
nv = size(T,2);
lambda=(exp(Ef)'*T)';
lambda = z.*lambda;
semilogy(reshape(lambda,nv,[]))
hold all, semilogy(reshape(y,nv,[]),'.')
legend('\lambda','y')
xlabel('Measurement index \it{i}')
ylabel('Number of particles counted/expected in each measurement')

% Examine predictions at a grid with one minute/ 5 s/ 10 s time-resolution
%hold on

step=0.1/log(10); %the resolution for the dp axis. Used also for the integration further down.
sizeaxis=min(x(:,2)):step:max(x(:,2));
%timeaxis=min(x(:,1)):1/(24*60):max(x(:,1));%one minute
timeaxis=min(x(:,1)):1/(12*24*60):max(x(:,1));% 5 s
%timeaxis=min(x(:,1)):1/(6*24*60):max(x(:,1));% 10 s
xt=reshape(repmat(timeaxis,length(sizeaxis),1),[],1);
repeat=length(xt)/length(sizeaxis);
xt(:,2)=repmat(sizeaxis',repeat,1);

[Eft, Varft] = gpla_predDMPS(gp,x,y,xt,'z',z);

% scatter(xt(:,1),exp10(xt(:,2)),3,Eft)
% set(gca,'Yscale','log')
% xlabel('Time')
% ylabel('Particle diameter [m]')




figure
h=surface(timeaxis,exp10(sizeaxis),reshape(Eft,length(sizeaxis),[]))
%h=surface(timeaxis,exp(sizeaxis),reshape(Eft/log(10),length(sizeaxis),[]))
set(h,'Edgecolor','none')
set(gca,'Yscale','log')
xlabel('Time [hh:mm]')
datetick('x')
%hold all
%plot3(x(:,1),exp(x(:,2)),Ef,'.k')
hc=colorbar;
ylabel('{\itD_p} [m]')
ylim([1e-8 1e-6])
%caxis([0 5])
%set(hc,'Ytick',0:5,'Yticklabel',{'1','10','100','1000','10000','100000'})
%text(32.582,2e-8,'d{\itN}/dlog_1_0({\itD_p}) [cm^{-3}]','Rotation',90)
title('E(f)')

figure
h=surface(timeaxis,exp10(sizeaxis),reshape(Varft,length(sizeaxis),[]))
set(h,'Edgecolor','none')
set(gca,'Yscale','log')
xlabel('Time [hh:mm]')
datetick('x')
hold all
plot3(x(:,1),exp10(x(:,2)),Ef,'.k')
title('Var(f)')
colorbar
ylabel('{\itD_p} [m]')
ylim([1e-8 1e-6])


figure
h=surf(timeaxis,exp10(sizeaxis),reshape(exp(Eft),length(sizeaxis),[]))
set(gca,'Yscale','log')
set(h,'Edgecolor','none')
zlabel('d{\itN}/dlog_1_0({\itD_p}) [cm^{-3}]')
xlabel('Time [hh:mm]')
datetick('x')
ylabel('{\itD_p} [m]');

%----------------------------
%Compare particle number concentrations from SMPS and CPC

figure

smps_sum=step*sum(exp(reshape(Eft+0.5*Varft,length(sizeaxis),[])));
plot(timeaxis,smps_sum)
hold all
%plot(reshape((repmat(dmpsOld(firstScan+(0:Nscan-1),1),1,3)+repmat([0 datenum(0,0,0,0,9,45) NaN],Nscan,1))',[],1),reshape([1;1;NaN]*dmpsOld(firstScan+(0:Nscan-1),2)',[],1))

%now load CPC data

cpc=load(['CH' int2str(year-2000) datestr(x(1),'mmdd') '.DA1'],'-ascii');
cpc(:,1)=floor(timeaxis(1))+cpc(:,1)/24; %changing time format
inTimeInterval=cpc(:,1)>timeaxis(1) & cpc(:,1)<timeaxis(end);

plot(cpc(inTimeInterval,1)-2.5/(24*3600),cpc(inTimeInterval,3))
%CPC time stamp are in the end of the averaging period. In the line above
%2.5 seconds were subtracted to get the middle of the averaging period.
legend('SMPS','CPC')
xlabel('Time [hh:mm]')
set(gca,'xtick',timeStampDOY(indexScans))
datetick('x',13,'keepTicks')
ylabel('PNC [cm^{-3}]')

%---------------------------
% sample from the posterior at time timeaxis(ii)
ii=100; %arbitrary choice. Change this index to choose the time you want.
xt=[ones(length(sizeaxis),1)*timeaxis(ii) sizeaxis']; 
fs = gpla_rndDMPS(gp,x,y,xt,'z',z, 'nsamp', 7);

figure
semilogx(exp10(sizeaxis),exp(fs))
ylabel('d{\itN}/dlog_1_0({\itD_p}) [cm^{-3}]')
xlabel('{\itD_p} [m]');
legend('Draw 1','Draw 2','Draw 3','Draw 4','Draw 5','Draw 6','Draw 7')
title(['Size distribution sampled from the posterior for time ' datestr(timeaxis(ii),13)])



fs = gpla_rndDMPS(gp,x,y,xt,'z',z, 'nsamp', 1e4);
dmps_sum_sample=step*sum(exp(fs));


